@@ -135,6 +135,10 @@ static GList *plugin_paths = NULL;      /* for delayed processing in post_init *
 
 extern gint _gst_trace_on;
 
+#ifndef GST_DISABLE_GST_DEBUG
+extern const gchar *priv_gst_dump_dot_dir;
+#endif
+
 /* defaults */
 #ifdef HAVE_FORK
 #define DEFAULT_FORK TRUE
@@ -142,9 +146,8 @@ extern gint _gst_trace_on;
 #define DEFAULT_FORK FALSE
 #endif /* HAVE_FORK */
 
-/* set to TRUE when segfaults need to be left as is, FIXME, this variable is
- * global. */
-gboolean _gst_disable_segtrap = FALSE;
+/* set to TRUE when segfaults need to be left as is */
+static gboolean _gst_disable_segtrap = FALSE;
 
 /* control the behaviour of registry rebuild */
 static gboolean _gst_enable_registry_fork = DEFAULT_FORK;
@@ -407,8 +410,6 @@ gst_init_check (int *argc, char **argv[], GError ** err)
   if (!g_thread_supported ())
     g_thread_init (NULL);
 
-  GST_INFO ("initializing GStreamer");
-
   if (gst_initialized) {
     GST_DEBUG ("already initialized gst");
     return TRUE;
@@ -588,6 +589,8 @@ init_pre (GOptionContext * context, GOptionGroup * group, gpointer data,
       parse_debug_list (debug_list);
     }
   }
+
+  priv_gst_dump_dot_dir = g_getenv ("GST_DEBUG_DUMP_DOT_DIR");
 #endif
   /* This is the earliest we can make stuff show up in the logs.
    * So give some useful info about GStreamer here */
@@ -713,7 +716,7 @@ scan_and_update_registry (GstRegistry * default_registry,
     g_free (home_plugins);
 
     /* add the main (installed) library path */
-    GST_DEBUG ("scanning home plugins %s", PLUGINDIR);
+    GST_DEBUG ("scanning main plugins %s", PLUGINDIR);
     changed |= gst_registry_scan_path (default_registry, PLUGINDIR);
   } else {
     gchar **list;
@@ -967,15 +970,15 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   _priv_gst_quarks_initialize ();
   _gst_format_initialize ();
   _gst_query_initialize ();
-  gst_object_get_type ();
-  gst_pad_get_type ();
-  gst_element_factory_get_type ();
-  gst_element_get_type ();
-  gst_type_find_factory_get_type ();
-  gst_bin_get_type ();
+  g_type_class_ref (gst_object_get_type ());
+  g_type_class_ref (gst_pad_get_type ());
+  g_type_class_ref (gst_element_factory_get_type ());
+  g_type_class_ref (gst_element_get_type ());
+  g_type_class_ref (gst_type_find_factory_get_type ());
+  g_type_class_ref (gst_bin_get_type ());
 
 #ifndef GST_DISABLE_INDEX
-  gst_index_factory_get_type ();
+  g_type_class_ref (gst_index_factory_get_type ());
 #endif /* GST_DISABLE_INDEX */
 #ifndef GST_DISABLE_URI
   gst_uri_handler_get_type ();
@@ -983,7 +986,7 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
 
   gst_structure_get_type ();
   _gst_value_initialize ();
-  gst_param_spec_fraction_get_type ();
+  g_type_class_ref (gst_param_spec_fraction_get_type ());
   gst_caps_get_type ();
   _gst_event_initialize ();
   _gst_buffer_initialize ();
@@ -1241,6 +1244,17 @@ gst_deinit (void)
 
   _priv_gst_registry_cleanup ();
 
+  g_type_class_unref (g_type_class_peek (gst_object_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pad_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_element_factory_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_element_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_type_find_factory_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_bin_get_type ()));
+#ifndef GST_DISABLE_INDEX
+  g_type_class_unref (g_type_class_peek (gst_index_factory_get_type ()));
+#endif /* GST_DISABLE_INDEX */
+  g_type_class_unref (g_type_class_peek (gst_param_spec_fraction_get_type ()));
+
   gst_initialized = FALSE;
   GST_INFO ("deinitialized GStreamer");
 }
