@@ -61,6 +61,8 @@
 #include "gsturi.h"
 #include "gstregistry.h"
 
+#include "glib-compat-private.h"
+
 GST_DEBUG_CATEGORY_STATIC (element_factory_debug);
 #define GST_CAT_DEFAULT element_factory_debug
 
@@ -217,10 +219,23 @@ gst_element_factory_cleanup (GstElementFactory * factory)
 
   for (item = factory->staticpadtemplates; item; item = item->next) {
     GstStaticPadTemplate *templ = item->data;
+    GstCaps *caps = (GstCaps *) & (templ->static_caps);
 
-    g_free (templ->name_template);
     g_free ((gchar *) templ->static_caps.string);
-    memset (&(templ->static_caps), 0, sizeof (GstStaticCaps));
+
+    /* FIXME: this is not threadsafe */
+    if (caps->refcount == 1) {
+      GstStructure *structure;
+      guint i;
+
+      for (i = 0; i < caps->structs->len; i++) {
+        structure = (GstStructure *) gst_caps_get_structure (caps, i);
+        gst_structure_set_parent_refcount (structure, NULL);
+        gst_structure_free (structure);
+      }
+      g_ptr_array_free (caps->structs, TRUE);
+      caps->refcount = 0;
+    }
     g_free (templ);
   }
   g_list_free (factory->staticpadtemplates);
@@ -279,7 +294,7 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
     GstStaticPadTemplate *newt;
 
     newt = g_new0 (GstStaticPadTemplate, 1);
-    newt->name_template = g_strdup (templ->name_template);
+    newt->name_template = g_intern_string (templ->name_template);
     newt->direction = templ->direction;
     newt->presence = templ->presence;
     newt->static_caps.string = gst_caps_to_string (templ->caps);
@@ -294,12 +309,19 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
     GstURIHandlerInterface *iface = (GstURIHandlerInterface *)
         g_type_interface_peek (klass, GST_TYPE_URI_HANDLER);
 
-    if (!iface || !iface->get_type || !iface->get_protocols)
+    if (!iface || (!iface->get_type && !iface->get_type_full) ||
+        (!iface->get_protocols && !iface->get_protocols_full))
       goto urierror;
-    factory->uri_type = iface->get_type ();
+    if (iface->get_type)
+      factory->uri_type = iface->get_type ();
+    else if (iface->get_type_full)
+      factory->uri_type = iface->get_type_full (factory->type);
     if (!GST_URI_TYPE_IS_VALID (factory->uri_type))
       goto urierror;
-    factory->uri_protocols = g_strdupv (iface->get_protocols ());
+    if (iface->get_protocols)
+      factory->uri_protocols = g_strdupv (iface->get_protocols ());
+    else if (iface->get_protocols_full)
+      factory->uri_protocols = iface->get_protocols_full (factory->type);
     if (!factory->uri_protocols)
       goto urierror;
   }
@@ -311,9 +333,9 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   g_free (interfaces);
 
   if (plugin && plugin->desc.name) {
-    GST_PLUGIN_FEATURE (factory)->plugin_name = g_strdup (plugin->desc.name);
+    GST_PLUGIN_FEATURE (factory)->plugin_name = plugin->desc.name;
   } else {
-    GST_PLUGIN_FEATURE (factory)->plugin_name = g_strdup ("NULL");
+    GST_PLUGIN_FEATURE (factory)->plugin_name = "NULL";
   }
   gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank);
   GST_PLUGIN_FEATURE (factory)->loaded = TRUE;
