@@ -125,13 +125,27 @@ read_string (xmlTextReaderPtr reader, gchar ** write_to, gboolean allow_blank)
     if (xmlTextReaderNodeType (reader) == XML_READER_TYPE_TEXT) {
       if (found)
         return FALSE;
-      *write_to = g_strdup ((gchar *) xmlTextReaderConstValue (reader));
+      *write_to = (gchar *) xmlTextReaderValue (reader);
       found = TRUE;
     }
   }
   return FALSE;
 }
 
+static gboolean
+read_const_interned_string (xmlTextReaderPtr reader, const gchar ** write_to,
+    gboolean allow_blank)
+{
+  gchar *s = NULL;
+
+  if (!read_string (reader, &s, allow_blank))
+    return FALSE;
+
+  *write_to = g_intern_string (s);
+  g_free (s);
+  return TRUE;
+}
+
 static gboolean
 read_uint (xmlTextReaderPtr reader, guint * write_to)
 {
@@ -200,7 +214,8 @@ load_pad_template (xmlTextReaderPtr reader)
 {
   int ret;
   int depth = xmlTextReaderDepth (reader);
-  gchar *name = NULL, *caps_str = NULL;
+  const gchar *name = NULL;
+  gchar *caps_str = NULL;
   guint direction = 0, presence = 0;
 
   while ((ret = xmlTextReaderRead (reader)) == 1) {
@@ -208,7 +223,7 @@ load_pad_template (xmlTextReaderPtr reader)
       GstStaticPadTemplate *template;
 
       template = g_new0 (GstStaticPadTemplate, 1);
-      template->name_template = name;
+      template->name_template = name;   /* must be an interned string! */
       template->presence = presence;
       template->direction = direction;
       template->static_caps.string = caps_str;
@@ -220,7 +235,7 @@ load_pad_template (xmlTextReaderPtr reader)
       const gchar *tag = (gchar *) xmlTextReaderConstName (reader);
 
       if (g_str_equal (tag, "nametemplate")) {
-        read_string (reader, &name, FALSE);
+        read_const_interned_string (reader, &name, FALSE);
       } else if (g_str_equal (tag, "direction")) {
         read_enum (reader, GST_TYPE_PAD_DIRECTION, &direction);
       } else if (g_str_equal (tag, "presence")) {
@@ -230,7 +245,6 @@ load_pad_template (xmlTextReaderPtr reader)
       }
     }
   }
-  g_free (name);
   g_free (caps_str);
 
   return NULL;
@@ -249,7 +263,7 @@ load_feature (xmlTextReaderPtr reader)
   feature_name =
       (gchar *) xmlTextReaderGetAttribute (reader, BAD_CAST "typename");
 
-  GST_DEBUG ("loading feature");
+  GST_LOG ("loading feature");
 
   if (!feature_name)
     return NULL;
@@ -275,7 +289,7 @@ load_feature (xmlTextReaderPtr reader)
   }
   while ((ret = xmlTextReaderRead (reader)) == 1) {
     if (xmlTextReaderDepth (reader) == depth) {
-      GST_DEBUG ("loaded feature %p with name %s", feature, feature->name);
+      GST_LOG ("loaded feature %p with name %s", feature, feature->name);
       return feature;
     }
     if (xmlTextReaderNodeType (reader) == XML_READER_TYPE_ELEMENT &&
@@ -284,17 +298,17 @@ load_feature (xmlTextReaderPtr reader)
 
       if (g_str_equal (tag, "name"))
         read_string (reader, &feature->name, FALSE);
-      if (g_str_equal (tag, "rank"))
+      else if (g_str_equal (tag, "rank"))
         read_uint (reader, &feature->rank);
+
       if (GST_IS_ELEMENT_FACTORY (feature)) {
-        GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
+        GstElementFactory *factory = GST_ELEMENT_FACTORY_CAST (feature);
 
         if (g_str_equal (tag, "longname")) {
           int ret;
 
           ret = read_string (reader, &factory->details.longname, TRUE);
-          GST_DEBUG ("longname ret=%d, name=%s",
-              ret, factory->details.longname);
+          GST_LOG ("longname ret=%d, name=%s", ret, factory->details.longname);
         } else if (g_str_equal (tag, "class")) {
           read_string (reader, &factory->details.klass, TRUE);
         } else if (g_str_equal (tag, "description")) {
@@ -351,11 +365,13 @@ load_feature (xmlTextReaderPtr reader)
             g_free (s);
           }
         }
+#ifndef GST_DISABLE_INDEX
       } else if (GST_IS_INDEX_FACTORY (feature)) {
         GstIndexFactory *factory = GST_INDEX_FACTORY (feature);
 
         if (g_str_equal (tag, "longdesc"))
           read_string (reader, &factory->longdesc, TRUE);
+#endif
       }
     }
   }
@@ -372,7 +388,7 @@ load_plugin (xmlTextReaderPtr reader, GList ** feature_list)
 
   *feature_list = NULL;
 
-  GST_DEBUG ("creating new plugin and parsing");
+  GST_LOG ("creating new plugin and parsing");
 
   plugin = g_object_new (GST_TYPE_PLUGIN, NULL);
 
@@ -388,86 +404,86 @@ load_plugin (xmlTextReaderPtr reader, GList ** feature_list)
       if (g_str_equal (tag, "name")) {
         int ret;
 
-        ret = read_string (reader, &plugin->desc.name, FALSE);
-        GST_DEBUG ("name ret=%d, name=%s", ret, plugin->desc.name);
+        ret = read_const_interned_string (reader, &plugin->desc.name, FALSE);
+        GST_LOG ("name ret=%d, name=%s", ret, plugin->desc.name);
         if (!ret)
           break;
       } else if (g_str_equal (tag, "description")) {
         if (!read_string (reader, &plugin->desc.description, TRUE)) {
-          GST_DEBUG ("description field was invalid in registry");
+          GST_WARNING ("description field was invalid in registry");
           break;
         }
-        GST_DEBUG ("description %s", plugin->desc.description);
+        GST_LOG ("description %s", plugin->desc.description);
       } else if (g_str_equal (tag, "filename")) {
         if (!read_string (reader, &plugin->filename, FALSE)) {
-          GST_DEBUG ("filename field was invalid in registry");
+          GST_WARNING ("filename field was invalid in registry");
           break;
         }
-        GST_DEBUG ("filename %s", plugin->filename);
+        GST_LOG ("filename %s", plugin->filename);
         plugin->basename = g_path_get_basename (plugin->filename);
       } else if (g_str_equal (tag, "version")) {
-        if (!read_string (reader, &plugin->desc.version, TRUE)) {
-          GST_DEBUG ("version field was invalid in registry");
+        if (!read_const_interned_string (reader, &plugin->desc.version, TRUE)) {
+          GST_WARNING ("version field was invalid in registry");
           break;
         }
-        GST_DEBUG ("version %s", plugin->desc.version);
+        GST_LOG ("version %s", plugin->desc.version);
       } else if (g_str_equal (tag, "license")) {
-        if (!read_string (reader, &plugin->desc.license, TRUE)) {
-          GST_DEBUG ("license field was invalid in registry");
+        if (!read_const_interned_string (reader, &plugin->desc.license, TRUE)) {
+          GST_WARNING ("license field was invalid in registry");
           break;
         }
-        GST_DEBUG ("license %s", plugin->desc.license);
+        GST_LOG ("license %s", plugin->desc.license);
       } else if (g_str_equal (tag, "source")) {
-        if (!read_string (reader, &plugin->desc.source, TRUE)) {
-          GST_DEBUG ("source field was invalid in registry");
+        if (!read_const_interned_string (reader, &plugin->desc.source, TRUE)) {
+          GST_WARNING ("source field was invalid in registry");
           break;
         }
-        GST_DEBUG ("source %s", plugin->desc.source);
+        GST_LOG ("source %s", plugin->desc.source);
       } else if (g_str_equal (tag, "package")) {
-        if (!read_string (reader, &plugin->desc.package, TRUE)) {
-          GST_DEBUG ("package field was invalid in registry");
+        if (!read_const_interned_string (reader, &plugin->desc.package, TRUE)) {
+          GST_WARNING ("package field was invalid in registry");
           break;
         }
-        GST_DEBUG ("package %s", plugin->desc.package);
+        GST_LOG ("package %s", plugin->desc.package);
       } else if (g_str_equal (tag, "origin")) {
-        if (!read_string (reader, &plugin->desc.origin, TRUE)) {
-          GST_DEBUG ("failed to read origin");
+        if (!read_const_interned_string (reader, &plugin->desc.origin, TRUE)) {
+          GST_WARNING ("failed to read origin");
           break;
         }
       } else if (g_str_equal (tag, "m32p")) {
         char *s;
 
         if (!read_string (reader, &s, FALSE)) {
-          GST_DEBUG ("failed to read mtime");
+          GST_WARNING ("failed to read mtime");
           break;
         }
         plugin->file_mtime = strtol (s, NULL, 0);
-        GST_DEBUG ("mtime %d", (int) plugin->file_mtime);
+        GST_LOG ("mtime %d", (int) plugin->file_mtime);
         g_free (s);
       } else if (g_str_equal (tag, "size")) {
         unsigned int x;
 
         if (read_uint (reader, &x)) {
           plugin->file_size = x;
-          GST_DEBUG ("file_size %d", plugin->file_size);
+          GST_LOG ("file_size %" G_GINT64_FORMAT, (gint64) plugin->file_size);
         } else {
-          GST_DEBUG ("failed to read size");
+          GST_WARNING ("failed to read size");
         }
       } else if (g_str_equal (tag, "feature")) {
         GstPluginFeature *feature = load_feature (reader);
 
         if (feature) {
-          feature->plugin_name = g_strdup (plugin->desc.name);
+          feature->plugin_name = plugin->desc.name;     /* interned string */
           *feature_list = g_list_prepend (*feature_list, feature);
         }
       } else {
-        GST_DEBUG ("unknown tag %s", tag);
+        GST_WARNING ("unknown tag %s", tag);
       }
     }
   }
   gst_object_unref (plugin);
 
-  GST_DEBUG ("problem reading plugin");
+  GST_WARNING ("problem reading plugin");
 
   return NULL;
 }
@@ -477,10 +493,9 @@ load_plugin (xmlTextReaderPtr reader, GList ** feature_list)
  * @registry: a #GstRegistry
  * @location: a filename
  *
- * Read the contents of the XML cache file at location 
- * @location into @registry.
+ * Read the contents of the XML cache file at @location into @registry.
  *
- * Returns: TRUE on success.
+ * Returns: %TRUE on success.
  */
 gboolean
 gst_registry_xml_read_cache (GstRegistry * registry, const char *location)
@@ -496,7 +511,9 @@ gst_registry_xml_read_cache (GstRegistry * registry, const char *location)
   /* make sure these types exist */
   GST_TYPE_ELEMENT_FACTORY;
   GST_TYPE_TYPE_FIND_FACTORY;
+#ifndef GST_DISABLE_INDEX
   GST_TYPE_INDEX_FACTORY;
+#endif
 
   timer = g_timer_new ();
 
@@ -541,10 +558,10 @@ gst_registry_xml_read_cache (GstRegistry * registry, const char *location)
           if (plugin) {
             GList *g;
 
-            GST_DEBUG ("adding plugin %s", plugin->desc.name);
             gst_registry_add_plugin (registry, plugin);
             for (g = feature_list; g; g = g_list_next (g)) {
-              gst_registry_add_feature (registry, GST_PLUGIN_FEATURE (g->data));
+              gst_registry_add_feature (registry,
+                  GST_PLUGIN_FEATURE_CAST (g->data));
             }
             g_list_free (feature_list);
           }
@@ -567,7 +584,7 @@ gst_registry_xml_read_cache (GstRegistry * registry, const char *location)
   seconds = g_timer_elapsed (timer, NULL);
   g_timer_destroy (timer);
 
-  GST_INFO ("loaded %s in %f seconds", location, seconds);
+  GST_INFO ("loaded %s in %lf seconds", location, seconds);
 
   if (mapped)
     g_mapped_file_free (mapped);
@@ -582,13 +599,20 @@ gst_registry_xml_read_cache (GstRegistry * registry, const char *location)
  * Save
  */
 static gboolean
-gst_registry_save_escaped (GstRegistry * registry, char *prefix, char *tag,
-    char *value)
+gst_registry_save_escaped (GstRegistry * registry, const char *prefix,
+    const char *tag, const char *value)
 {
   gboolean ret = TRUE;
 
   if (value) {
-    gchar *v = g_markup_escape_text (value, strlen (value));
+    gchar *v;
+
+    if (g_utf8_validate (value, -1, NULL)) {
+      v = g_markup_escape_text (value, -1);
+    } else {
+      g_warning ("Invalid UTF-8 while saving registry tag '%s'", tag);
+      v = g_strdup ("[ERROR: invalid UTF-8]");
+    }
 
     ret = gst_registry_save (registry, "%s<%s>%s</%s>\n", prefix, tag, v, tag);
     g_free (v);
@@ -711,18 +735,22 @@ gst_registry_xml_save_feature (GstRegistry * registry,
     }
 
     if (GST_URI_TYPE_IS_VALID (factory->uri_type)) {
-      gchar **protocol;
-
       if (!gst_registry_save_escaped (registry, "  ", "uri_type",
               factory->uri_type == GST_URI_SINK ? "sink" : "source"))
         return FALSE;
-      g_assert (factory->uri_protocols);
-      protocol = factory->uri_protocols;
-      while (*protocol) {
-        if (!gst_registry_save_escaped (registry, "  ", "uri_protocol",
-                *protocol))
-          return FALSE;
-        protocol++;
+      if (factory->uri_protocols) {
+        gchar **protocol;
+
+        protocol = factory->uri_protocols;
+        while (*protocol) {
+          if (!gst_registry_save_escaped (registry, "  ", "uri_protocol",
+                  *protocol))
+            return FALSE;
+          protocol++;
+        }
+      } else {
+        g_warning ("GStreamer feature '%s' is URI handler but does not provide"
+            " any protocols it can handle", feature->name);
       }
     }
   } else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
@@ -741,10 +769,12 @@ gst_registry_xml_save_feature (GstRegistry * registry,
         i++;
       }
     }
+#ifndef GST_DISABLE_INDEX
   } else if (GST_IS_INDEX_FACTORY (feature)) {
     if (!gst_registry_save_escaped (registry, "  ", "longdesc",
             GST_INDEX_FACTORY (feature)->longdesc))
       return FALSE;
+#endif
   }
   return TRUE;
 }
@@ -838,11 +868,16 @@ gst_registry_xml_write_cache (GstRegistry * registry, const char *location)
     g_mkdir_with_parents (dir, 0777);
     g_free (dir);
 
-    registry->cache_file = g_mkstemp (tmp_location);
-  }
-  if (registry->cache_file == -1) {
+    /* the previous g_mkstemp call overwrote the XXXXXX placeholder ... */
     g_free (tmp_location);
-    return FALSE;
+    tmp_location = g_strconcat (location, ".tmpXXXXXX", NULL);
+    registry->cache_file = g_mkstemp (tmp_location);
+
+    if (registry->cache_file == -1) {
+      GST_DEBUG ("g_mkstemp() failed: %s", g_strerror (errno));
+      g_free (tmp_location);
+      return FALSE;
+    }
   }
 
   if (!gst_registry_save (registry, "<?xml version=\"1.0\"?>\n"))
@@ -853,7 +888,7 @@ gst_registry_xml_write_cache (GstRegistry * registry, const char *location)
 
   for (walk = g_list_last (registry->plugins); walk;
       walk = g_list_previous (walk)) {
-    GstPlugin *plugin = GST_PLUGIN (walk->data);
+    GstPlugin *plugin = GST_PLUGIN_CAST (walk->data);
 
     if (!plugin->filename)
       continue;
