@@ -55,15 +55,17 @@ static void gst_system_clock_dispose (GObject * object);
 
 static GstClockTime gst_system_clock_get_internal_time (GstClock * clock);
 static guint64 gst_system_clock_get_resolution (GstClock * clock);
-static GstClockReturn gst_system_clock_id_wait (GstClock * clock,
-    GstClockEntry * entry);
-static GstClockReturn gst_system_clock_id_wait_unlocked
-    (GstClock * clock, GstClockEntry * entry);
+static GstClockReturn gst_system_clock_id_wait_jitter (GstClock * clock,
+    GstClockEntry * entry, GstClockTimeDiff * jitter);
+static GstClockReturn gst_system_clock_id_wait_jitter_unlocked
+    (GstClock * clock, GstClockEntry * entry, GstClockTimeDiff * jitter,
+    gboolean restart);
 static GstClockReturn gst_system_clock_id_wait_async (GstClock * clock,
     GstClockEntry * entry);
 static void gst_system_clock_id_unschedule (GstClock * clock,
     GstClockEntry * entry);
 static void gst_system_clock_async_thread (GstClock * clock);
+static gboolean gst_system_clock_start_async (GstSystemClock * clock);
 
 static GStaticMutex _gst_sysclock_mutex = G_STATIC_MUTEX_INIT;
 
@@ -113,7 +115,7 @@ gst_system_clock_class_init (GstSystemClockClass * klass)
 
   gstclock_class->get_internal_time = gst_system_clock_get_internal_time;
   gstclock_class->get_resolution = gst_system_clock_get_resolution;
-  gstclock_class->wait = gst_system_clock_id_wait;
+  gstclock_class->wait_jitter = gst_system_clock_id_wait_jitter;
   gstclock_class->wait_async = gst_system_clock_id_wait_async;
   gstclock_class->unschedule = gst_system_clock_id_unschedule;
 }
@@ -121,31 +123,18 @@ gst_system_clock_class_init (GstSystemClockClass * klass)
 static void
 gst_system_clock_init (GstSystemClock * clock)
 {
-  GError *error = NULL;
-
   GST_OBJECT_FLAG_SET (clock,
       GST_CLOCK_FLAG_CAN_DO_SINGLE_SYNC |
       GST_CLOCK_FLAG_CAN_DO_SINGLE_ASYNC |
       GST_CLOCK_FLAG_CAN_DO_PERIODIC_SYNC |
       GST_CLOCK_FLAG_CAN_DO_PERIODIC_ASYNC);
 
-  GST_OBJECT_LOCK (clock);
-  clock->thread = g_thread_create ((GThreadFunc) gst_system_clock_async_thread,
-      clock, TRUE, &error);
-  if (error)
-    goto no_thread;
-
-  /* wait for it to spin up */
-  GST_CLOCK_WAIT (clock);
-  GST_OBJECT_UNLOCK (clock);
-  return;
-
-  /* ERRORS */
-no_thread:
-  {
-    g_warning ("could not create async clock thread: %s", error->message);
-    GST_OBJECT_UNLOCK (clock);
-  }
+#if 0
+  /* Uncomment this to start the async clock thread straight away */
+  GST_CLOCK_LOCK (clock);
+  gst_system_clock_start_async (clock);
+  GST_CLOCK_UNLOCK (clock);
+#endif
 }
 
 static void
@@ -229,7 +218,8 @@ gst_system_clock_obtain (void)
  *
  * It waits on each of them and fires the callback when the timeout occurs.
  *
- * When an entry in the queue was canceled, it is simply skipped.
+ * When an entry in the queue was canceled before we wait for it, it is
+ * simply skipped.
  *
  * When waiting for an entry, it can become canceled, in that case we don't
  * call the callback but move to the next item in the queue.
@@ -248,6 +238,7 @@ gst_system_clock_async_thread (GstClock * clock)
   /* now enter our (almost) infinite loop */
   while (!sysclock->stopping) {
     GstClockEntry *entry;
+    GstClockTime requested;
     GstClockReturn res;
 
     /* check if something to be done */
@@ -269,8 +260,12 @@ gst_system_clock_async_thread (GstClock * clock)
       goto next_entry;
     }
 
+    requested = entry->time;
+
     /* now wait for the entry, we already hold the lock */
-    res = gst_system_clock_id_wait_unlocked (clock, (GstClockID) entry);
+    res =
+        gst_system_clock_id_wait_jitter_unlocked (clock, (GstClockID) entry,
+        NULL, FALSE);
 
     switch (res) {
       case GST_CLOCK_UNSCHEDULED:
@@ -292,7 +287,7 @@ gst_system_clock_async_thread (GstClock * clock)
         }
         if (entry->type == GST_CLOCK_ENTRY_PERIODIC) {
           /* adjust time now */
-          entry->time += entry->interval;
+          entry->time = requested + entry->interval;
           /* and resort the list now */
           clock->entries =
               g_list_sort (clock->entries, gst_clock_id_compare_func);
@@ -362,7 +357,8 @@ gst_system_clock_get_resolution (GstClock * clock)
  * MT safe.
  */
 static GstClockReturn
-gst_system_clock_id_wait_unlocked (GstClock * clock, GstClockEntry * entry)
+gst_system_clock_id_wait_jitter_unlocked (GstClock * clock,
+    GstClockEntry * entry, GstClockTimeDiff * jitter, gboolean restart)
 {
   GstClockTime entryt, real, now, target;
   GstClockTimeDiff diff;
@@ -372,7 +368,15 @@ gst_system_clock_id_wait_unlocked (GstClock * clock, GstClockEntry * entry)
   entryt = GST_CLOCK_ENTRY_TIME (entry);
 
   now = gst_clock_adjust_unlocked (clock, real);
+  if (jitter) {
+    *jitter = GST_CLOCK_DIFF (entryt, now);
+  }
   diff = entryt - now;
+  /* FIXME: should that be not just:
+   * target = GST_CLOCK_GET_CLASS (clock)->get_internal_time (clock) + diff;
+   * or even
+   * target = real + diff;
+   */
   target = gst_system_clock_get_internal_time (clock) + diff;
 
   GST_CAT_DEBUG (GST_CAT_CLOCK, "entry %p"
@@ -380,7 +384,7 @@ gst_system_clock_id_wait_unlocked (GstClock * clock, GstClockEntry * entry)
       " entry %" GST_TIME_FORMAT
       " now %" GST_TIME_FORMAT
       " real %" GST_TIME_FORMAT
-      " diff %" G_GINT64_FORMAT,
+      " diff (entry-now) %" G_GINT64_FORMAT,
       entry,
       GST_TIME_ARGS (target),
       GST_TIME_ARGS (entryt), GST_TIME_ARGS (now), GST_TIME_ARGS (real), diff);
@@ -413,15 +417,22 @@ gst_system_clock_id_wait_unlocked (GstClock * clock, GstClockEntry * entry)
             (final - target),
             ((double) (GstClockTimeDiff) (final - target)) / GST_SECOND);
 #endif
-
         break;
       } else {
         /* the waiting is interrupted because the GCond was signaled. This can
          * be because this or some other entry was unscheduled. */
         GST_CAT_DEBUG (GST_CAT_CLOCK, "entry %p unlocked with signal", entry);
-        /* if the entry is unscheduled, we can stop waiting for it */
+        /* if the entry is unscheduled, we can stop waiting for it, else we
+         * continue our while loop. */
         if (entry->status == GST_CLOCK_UNSCHEDULED)
           break;
+        /* else restart if we must */
+        if (!restart)
+          break;
+
+        /* this can happen if the entry got unlocked because of an async entry
+         * was added to the head of the async queue. */
+        GST_CAT_DEBUG (GST_CAT_CLOCK, "continue waiting for entry %p", entry);
       }
     }
   } else if (diff == 0) {
@@ -433,17 +444,46 @@ gst_system_clock_id_wait_unlocked (GstClock * clock, GstClockEntry * entry)
 }
 
 static GstClockReturn
-gst_system_clock_id_wait (GstClock * clock, GstClockEntry * entry)
+gst_system_clock_id_wait_jitter (GstClock * clock, GstClockEntry * entry,
+    GstClockTimeDiff * jitter)
 {
   GstClockReturn ret;
 
   GST_OBJECT_LOCK (clock);
-  ret = gst_system_clock_id_wait_unlocked (clock, entry);
+  ret = gst_system_clock_id_wait_jitter_unlocked (clock, entry, jitter, TRUE);
   GST_OBJECT_UNLOCK (clock);
 
   return ret;
 }
 
+/* Start the async clock thread. Must be called with the object lock
+ * held */
+static gboolean
+gst_system_clock_start_async (GstSystemClock * clock)
+{
+  GError *error = NULL;
+
+  if (clock->thread != NULL)
+    return TRUE;                /* Thread already running. Nothing to do */
+
+  clock->thread = g_thread_create ((GThreadFunc) gst_system_clock_async_thread,
+      clock, TRUE, &error);
+  if (error)
+    goto no_thread;
+
+  /* wait for it to spin up */
+  GST_CLOCK_WAIT (clock);
+
+  return TRUE;
+
+  /* ERRORS */
+no_thread:
+  {
+    g_warning ("could not create async clock thread: %s", error->message);
+  }
+  return FALSE;
+}
+
 /* Add an entry to the list of pending async waits. The entry is inserted
  * in sorted order. If we inserted the entry at the head of the list, we
  * need to signal the thread as it might either be waiting on it or waiting
@@ -454,9 +494,14 @@ gst_system_clock_id_wait (GstClock * clock, GstClockEntry * entry)
 static GstClockReturn
 gst_system_clock_id_wait_async (GstClock * clock, GstClockEntry * entry)
 {
-  GST_CAT_DEBUG (GST_CAT_CLOCK, "adding entry %p", entry);
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "adding async entry %p", entry);
 
   GST_OBJECT_LOCK (clock);
+
+  /* Start the clock async thread if needed */
+  if (!gst_system_clock_start_async (GST_SYSTEM_CLOCK (clock)))
+    goto thread_error;
+
   /* need to take a ref */
   gst_clock_id_ref ((GstClockID) entry);
   /* insert the entry in sorted order */
@@ -467,12 +512,21 @@ gst_system_clock_id_wait_async (GstClock * clock, GstClockEntry * entry)
    * front, else the thread is just waiting for another entry and
    * will get to this entry automatically. */
   if (clock->entries->data == entry) {
-    GST_CAT_DEBUG (GST_CAT_CLOCK, "send signal");
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "async entry added to head, sending signal");
+    /* this will wake up _all_ entries waiting for the clock because we have
+     * only one cond for all entries (makes allocation faster). Entries that
+     * have not timed out will have their status set to BUSY and should continue
+     * to wait. In the case of the async ones, the new head entry should be
+     * taken and waited for. */
     GST_CLOCK_BROADCAST (clock);
   }
   GST_OBJECT_UNLOCK (clock);
 
   return GST_CLOCK_OK;
+
+thread_error:
+  /* Could not start the async clock thread */
+  return GST_CLOCK_ERROR;
 }
 
 /* unschedule an entry. This will set the state of the entry to GST_CLOCK_UNSCHEDULED
@@ -488,8 +542,11 @@ gst_system_clock_id_unschedule (GstClock * clock, GstClockEntry * entry)
   GST_CAT_DEBUG (GST_CAT_CLOCK, "unscheduling entry %p", entry);
 
   GST_OBJECT_LOCK (clock);
+  /* mark entry as unscheduled, then wake up all entries. The entries that did
+   * not timeout will be woken up but immediatly go to sleep again because their
+   * status would still be busy. */
   entry->status = GST_CLOCK_UNSCHEDULED;
-  GST_CAT_DEBUG (GST_CAT_CLOCK, "send signal");
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "sending signal");
   GST_CLOCK_BROADCAST (clock);
   GST_OBJECT_UNLOCK (clock);
 }
